@@ -50,6 +50,13 @@ G_DEFINE_DYNAMIC_TYPE (CcPrintersPanel, cc_printers_panel, CC_TYPE_PANEL)
 #define CLOCK_SCHEMA "org.gnome.desktop.interface"
 #define CLOCK_FORMAT_KEY "clock-format"
 
+#define RENEW_INTERVAL        500
+#define SUBSCRIPTION_DURATION 600
+
+#define CUPS_DBUS_NAME      "org.cups.cupsd.Notifier"
+#define CUPS_DBUS_PATH      "/org/cups/cupsd/Notifier"
+#define CUPS_DBUS_INTERFACE "org.cups.cupsd.Notifier"
+
 struct _CcPrintersPanelPrivate
 {
   GtkBuilder *builder;
@@ -77,14 +84,20 @@ struct _CcPrintersPanelPrivate
 
   PpNewPrinterDialog *pp_new_printer_dialog;
 
+  GDBusProxy      *cups_proxy;
+  GDBusConnection *cups_bus_connection;
+  gint             subscription_id;
+
   gpointer dummy;
 };
 
 static void actualize_jobs_list (CcPrintersPanel *self);
 static void actualize_printers_list (CcPrintersPanel *self);
 static void actualize_allowed_users_list (CcPrintersPanel *self);
+static void actualize_sensitivity (gpointer user_data);
 static void printer_disable_cb (GObject *gobject, GParamSpec *pspec, gpointer user_data);
 static void printer_set_default_cb (GtkToggleButton *button, gpointer user_data);
+static void detach_from_cups_notifier (gpointer data);
 
 static void
 cc_printers_panel_get_property (GObject    *object,
@@ -157,6 +170,8 @@ cc_printers_panel_dispose (GObject *object)
       priv->lockdown_settings = NULL;
     }
 
+  detach_from_cups_notifier (CC_PRINTERS_PANEL (object));
+
   G_OBJECT_CLASS (cc_printers_panel_parent_class)->dispose (object);
 }
 
@@ -184,6 +199,199 @@ cc_printers_panel_class_finalize (CcPrintersPanelClass *klass)
 {
 }
 
+static void
+on_cups_notification (GDBusConnection *connection,
+                      const char      *sender_name,
+                      const char      *object_path,
+                      const char      *interface_name,
+                      const char      *signal_name,
+                      GVariant        *parameters,
+                      gpointer         user_data)
+{
+  CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
+  CcPrintersPanelPrivate *priv;
+  gboolean                printer_is_accepting_jobs;
+  gchar                  *printer_name = NULL;
+  gchar                  *text = NULL;
+  gchar                  *printer_uri = NULL;
+  gchar                  *printer_state_reasons = NULL;
+  gchar                  *job_state_reasons = NULL;
+  gchar                  *job_name = NULL;
+  guint                   job_id;
+  gint                    printer_state;
+  gint                    job_state;
+  gint                    job_impressions_completed;
+  static const char * const requested_attrs[] = {
+    "job-printer-uri",
+    "job-originating-user-name"};
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  if (g_strcmp0 (signal_name, "PrinterAdded") != 0 &&
+      g_strcmp0 (signal_name, "PrinterDeleted") != 0 &&
+      g_strcmp0 (signal_name, "PrinterStateChanged") != 0 &&
+      g_strcmp0 (signal_name, "JobCreated") != 0 &&
+      g_strcmp0 (signal_name, "JobCompleted") != 0)
+    return;
+
+  if (g_variant_n_children (parameters) == 1)
+    g_variant_get (parameters, "(&s)", &text);
+  else if (g_variant_n_children (parameters) == 6)
+    {
+      g_variant_get (parameters, "(&s&s&su&sb)",
+                     &text,
+                     &printer_uri,
+                     &printer_name,
+                     &printer_state,
+                     &printer_state_reasons,
+                     &printer_is_accepting_jobs);
+    }
+  else if (g_variant_n_children (parameters) == 11)
+    {
+      g_variant_get (parameters, "(&s&s&su&sbuu&s&su)",
+                     &text,
+                     &printer_uri,
+                     &printer_name,
+                     &printer_state,
+                     &printer_state_reasons,
+                     &printer_is_accepting_jobs,
+                     &job_id,
+                     &job_state,
+                     &job_state_reasons,
+                     &job_name,
+                     &job_impressions_completed);
+    }
+
+  if (g_strcmp0 (signal_name, "PrinterAdded") == 0 ||
+      g_strcmp0 (signal_name, "PrinterDeleted") == 0 ||
+      g_strcmp0 (signal_name, "PrinterStateChanged") == 0)
+    actualize_printers_list (self);
+  else if (g_strcmp0 (signal_name, "JobCreated") == 0 ||
+           g_strcmp0 (signal_name, "JobCompleted") == 0)
+    {
+      http_t *http;
+      gchar  *job_uri;
+      ipp_t  *request, *response;
+
+      job_uri = g_strdup_printf ("ipp://localhost/jobs/%d", job_id);
+      if ((http = httpConnectEncrypt (cupsServer (), ippPort (),
+                                     cupsEncryption ())) != NULL)
+        {
+          request = ippNewRequest(IPP_GET_JOB_ATTRIBUTES);
+          ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
+                       "job-uri", NULL, job_uri);
+          ippAddStrings (request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
+                         "requested-attributes", G_N_ELEMENTS (requested_attrs), NULL, requested_attrs);
+          response = cupsDoRequest(http, request, "/");
+
+          if (response)
+            {
+              if (response->request.status.status_code <= IPP_OK_CONFLICT)
+                {
+                  ipp_attribute_t *attr_username = NULL;
+                  ipp_attribute_t *attr_printer_uri = NULL;
+
+                  attr_username = ippFindAttribute(response, "job-originating-user-name", IPP_TAG_NAME);
+                  attr_printer_uri = ippFindAttribute(response, "job-printer-uri", IPP_TAG_URI);
+                  if (attr_username && attr_printer_uri &&
+                      g_strcmp0 (attr_username->values[0].string.text, cupsUser ()) == 0 &&
+                      g_strrstr (attr_printer_uri->values[0].string.text, "/") != 0 &&
+                      priv->current_dest >= 0 &&
+                      priv->current_dest < priv->num_dests &&
+                      priv->dests != NULL &&
+                      g_strcmp0 (g_strrstr (attr_printer_uri->values[0].string.text, "/") + 1,
+                                 priv->dests[priv->current_dest].name) == 0)
+                    actualize_jobs_list (self);
+                }
+              ippDelete(response);
+            }
+          httpClose (http);
+        }
+      g_free (job_uri);
+    }
+}
+
+static gboolean
+renew_subscription (gpointer data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) data;
+  static const char * const events[] = {
+          "printer-added",
+          "printer-deleted",
+          "printer-state-changed",
+          "job-created",
+          "job-completed"};
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  priv->subscription_id = renew_cups_subscription (priv->subscription_id,
+                                                   events,
+                                                   G_N_ELEMENTS (events),
+                                                   SUBSCRIPTION_DURATION);
+
+  return TRUE;
+}
+
+static void
+attach_to_cups_notifier (gpointer data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) data;
+  GError                 *error = NULL;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  renew_subscription (self);
+  g_timeout_add_seconds (RENEW_INTERVAL, renew_subscription, self);
+
+  error = NULL;
+  priv->cups_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                                    0,
+                                                    NULL,
+                                                    CUPS_DBUS_NAME,
+                                                    CUPS_DBUS_PATH,
+                                                    CUPS_DBUS_INTERFACE,
+                                                    NULL,
+                                                    &error);
+
+  if (error)
+    {
+      g_warning ("%s", error->message);
+      return;
+    }
+
+  priv->cups_bus_connection = g_dbus_proxy_get_connection (priv->cups_proxy);
+
+  g_dbus_connection_signal_subscribe (priv->cups_bus_connection,
+                                      NULL,
+                                      CUPS_DBUS_INTERFACE,
+                                      NULL,
+                                      CUPS_DBUS_PATH,
+                                      NULL,
+                                      0,
+                                      on_cups_notification,
+                                      self,
+                                      NULL);
+}
+
+static void
+detach_from_cups_notifier (gpointer data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) data;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  cancel_cups_subscription (priv->subscription_id);
+  priv->subscription_id = -1;
+
+  if (priv->cups_proxy != NULL) {
+    g_object_unref (priv->cups_proxy);
+    priv->cups_proxy = NULL;
+  }
+}
+
 enum
 {
   NOTEBOOK_INFO_PAGE = 0,
@@ -211,16 +419,14 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
   CcPrintersPanelPrivate *priv;
   CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
   GtkTreeModel           *model;
+  cups_ptype_t            type = 0;
   GtkTreeIter             iter;
   GtkWidget              *widget;
-  gboolean                test_page_command_available = FALSE;
   gboolean                sensitive;
   gboolean                is_local = TRUE;
   gchar                  *printer_make_and_model = NULL;
   gchar                  *printer_model = NULL;
   gchar                  *reason = NULL;
-  gchar                 **available_commands = NULL;
-  gchar                  *printer_commands = NULL;
   gchar                 **printer_reasons = NULL;
   gchar                  *marker_types = NULL;
   gchar                  *printer_name = NULL;
@@ -327,8 +533,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
             reason = priv->dests[id].options[i].value;
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "marker-types") == 0)
             marker_types = priv->dests[priv->current_dest].options[i].value;
-          else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-commands") == 0)
-            printer_commands = priv->dests[priv->current_dest].options[i].value;
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-make-and-model") == 0)
             printer_make_and_model = priv->dests[priv->current_dest].options[i].value;
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-uri-supported") == 0)
@@ -500,7 +704,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
 
       if (printer_type)
         {
-          cups_ptype_t type;
           type = atoi (printer_type);
           is_local = !(type & (CUPS_PRINTER_REMOTE | CUPS_PRINTER_IMPLICIT));
         }
@@ -551,22 +754,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       gtk_widget_set_sensitive (widget, sensitive);
 
 
-      if (printer_commands)
-        {
-          available_commands = g_strsplit (printer_commands, ",", -1);
-          for (i = 0; i < g_strv_length (available_commands); i++)
-            {
-              if (g_strcmp0 (available_commands[i], "PrintSelfTestPage") == 0)
-                test_page_command_available = TRUE;
-            }
-          g_strfreev (available_commands);
-        }
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "print-test-page-button");
-      gtk_widget_set_sensitive (widget, test_page_command_available);
-
-
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "supply-drawing-area");
       gtk_widget_set_size_request (widget, -1, SUPPLY_BAR_HEIGHT);
@@ -608,19 +795,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
         }
       else
         gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
-
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-options-button");
-      gtk_widget_set_sensitive (widget, TRUE);
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-jobs-button");
-      gtk_widget_set_sensitive (widget, TRUE);
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-icon");
-      gtk_widget_set_sensitive (widget, TRUE);
     }
   else
     {
@@ -661,24 +835,10 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-jobs-label");
       gtk_label_set_text (GTK_LABEL (widget), "");
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "print-test-page-button");
-      gtk_widget_set_sensitive (widget, FALSE);
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-options-button");
-      gtk_widget_set_sensitive (widget, FALSE);
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-jobs-button");
-      gtk_widget_set_sensitive (widget, FALSE);
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-icon");
-      gtk_widget_set_sensitive (widget, FALSE);
     }
 
+  actualize_sensitivity (self);
+
   widget = (GtkWidget*)
     gtk_builder_get_object (priv->builder, "job-release-button");
   gtk_widget_set_sensitive (widget, FALSE);
@@ -990,7 +1150,10 @@ actualize_jobs_list (CcPrintersPanel *self)
   GtkTreeView            *treeview;
   GtkTreeIter             iter;
   GSettings              *settings;
-  int                     i;
+  GtkWidget              *widget;
+  gchar                  *active_jobs;
+  gint                    num_jobs;
+  gint                    i;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
@@ -1006,7 +1169,24 @@ actualize_jobs_list (CcPrintersPanel *self)
   if (priv->current_dest >= 0 &&
       priv->current_dest < priv->num_dests &&
       priv->dests != NULL)
-    priv->num_jobs = cupsGetJobs (&priv->jobs, priv->dests[priv->current_dest].name, 1, CUPS_WHICHJOBS_ACTIVE);
+    {
+      priv->num_jobs = cupsGetJobs (&priv->jobs, priv->dests[priv->current_dest].name, 1, CUPS_WHICHJOBS_ACTIVE);
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-jobs-label");
+
+      num_jobs = priv->num_jobs < 0 ? 0 : (guint) priv->num_jobs;
+      /* Translators: there is n active print jobs on this printer */
+      active_jobs = g_strdup_printf (ngettext ("%u active", "%u active", num_jobs), num_jobs);
+
+      if (active_jobs)
+        {
+          gtk_label_set_text (GTK_LABEL (widget), active_jobs);
+          g_free (active_jobs);
+        }
+      else
+        gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
+    }
 
   store = gtk_list_store_new (JOB_N_COLUMNS, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
@@ -1961,50 +2141,6 @@ printer_remove_cb (GtkToolButton *toolbutton,
   }
 }
 
-static void
-printer_maintenance_cb (GtkButton *button,
-                        gpointer   user_data)
-{
-  CcPrintersPanelPrivate  *priv;
-  CcPrintersPanel         *self = (CcPrintersPanel*) user_data;
-  ipp_t                   *response = NULL;
-  gchar                   *printer_name = NULL;
-
-  priv = PRINTERS_PANEL_PRIVATE (self);
-
-  if (priv->current_dest >= 0 &&
-      priv->current_dest < priv->num_dests &&
-      priv->dests != NULL)
-    printer_name = priv->dests[priv->current_dest].name;
-
-  if (printer_name)
-    {
-      if ((GtkButton*) gtk_builder_get_object (priv->builder,
-                                               "print-test-page-button")
-                       == button)
-        {
-          response = execute_maintenance_command (printer_name,
-                                                  "PrintSelfTestPage",
-          /* Translators: Name of job which makes printer to print test page */
-                                                  _("Test page"));
-        }
-      else if ((GtkButton*) gtk_builder_get_object (priv->builder,
-                                                    "clean-print-heads-button")
-                            == button)
-        response = execute_maintenance_command (printer_name,
-                                                "Clean all",
-          /* Translators: Name of job which makes printer to clean its heads */
-                                                _("Clean print heads"));
-      if (response)
-        {
-          if (response->state == IPP_ERROR)
-            /* Translators: An error has occured during execution of a CUPS maintenance command */
-            g_warning (_("An error has occured during a maintenance command."));
-          ippDelete(response);
-        }
-    }
-}
-
 static void
 test_page_cb (GtkButton *button,
               gpointer   user_data)
@@ -2129,20 +2265,40 @@ test_page_cb (GtkButton *button,
 }
 
 static void
-on_permission_changed (GPermission *permission,
-                       GParamSpec  *pspec,
-                       gpointer     data)
+actualize_sensitivity (gpointer user_data)
 {
   CcPrintersPanelPrivate  *priv;
-  CcPrintersPanel         *self = (CcPrintersPanel*) data;
+  CcPrintersPanel         *self = (CcPrintersPanel*) user_data;
+  cups_ptype_t             type = 0;
   GtkWidget               *widget;
   gboolean                 is_authorized;
+  gboolean                 is_discovered = FALSE;
+  gboolean                 printer_selected;
+  gint                     i;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
-  is_authorized = g_permission_get_allowed (G_PERMISSION (priv->permission)) &&
+  is_authorized =
+    priv->permission &&
+    g_permission_get_allowed (G_PERMISSION (priv->permission)) &&
+    priv->lockdown_settings &&
     !g_settings_get_boolean (priv->lockdown_settings, "disable-print-setup");
 
+  printer_selected = priv->current_dest >= 0 &&
+                     priv->current_dest < priv->num_dests &&
+                     priv->dests != NULL;
+
+  if (printer_selected)
+    for (i = 0; i < priv->dests[priv->current_dest].num_options; i++)
+      {
+        if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-type") == 0)
+          {
+            type = atoi (priv->dests[priv->current_dest].options[i].value);
+            is_discovered = type & CUPS_PRINTER_DISCOVERED;
+            break;
+          }
+      }
+
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-add-button");
   gtk_widget_set_sensitive (widget, is_authorized);
 
@@ -2150,7 +2306,7 @@ on_permission_changed (GPermission *permission,
   gtk_widget_set_sensitive (widget, is_authorized);
 
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-remove-button");
-  gtk_widget_set_sensitive (widget, is_authorized);
+  gtk_widget_set_sensitive (widget, !is_discovered && is_authorized);
 
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-disable-switch");
   gtk_widget_set_sensitive (widget, is_authorized);
@@ -2163,6 +2319,26 @@ on_permission_changed (GPermission *permission,
 
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "allowed-user-remove-button");
   gtk_widget_set_sensitive (widget, is_authorized);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "print-test-page-button");
+  gtk_widget_set_sensitive (widget, printer_selected);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-options-button");
+  gtk_widget_set_sensitive (widget, printer_selected);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-jobs-button");
+  gtk_widget_set_sensitive (widget, printer_selected);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-icon");
+  gtk_widget_set_sensitive (widget, printer_selected);
+}
+
+static void
+on_permission_changed (GPermission *permission,
+                       GParamSpec  *pspec,
+                       gpointer     data)
+{
+  actualize_sensitivity (data);
 }
 
 static void
@@ -2259,6 +2435,14 @@ cc_printers_panel_init (CcPrintersPanel *self)
 
   priv->pp_new_printer_dialog = NULL;
 
+  priv->subscription_id = -1;
+  priv->cups_proxy = NULL;
+  priv->cups_bus_connection = NULL;
+
+  priv->lock_button = NULL;
+  priv->permission = NULL;
+  priv->lockdown_settings = NULL;
+
   gtk_builder_add_objects_from_file (priv->builder,
                                      DATADIR"/printers.ui",
                                      objects, &error);
@@ -2341,10 +2525,11 @@ cc_printers_panel_init (CcPrintersPanel *self)
   g_signal_connect (widget, "clicked", G_CALLBACK (switch_to_options_cb), self);
 
   priv->lockdown_settings = g_settings_new ("org.gnome.desktop.lockdown");
-  g_signal_connect (priv->lockdown_settings,
-                    "changed",
-                    G_CALLBACK (on_lockdown_settings_changed),
-                    self);
+  if (priv->lockdown_settings)
+    g_signal_connect (priv->lockdown_settings,
+                      "changed",
+                      G_CALLBACK (on_lockdown_settings_changed),
+                      self);
 
 
   /* Set junctions */
@@ -2394,6 +2579,10 @@ cc_printers_panel_init (CcPrintersPanel *self)
       on_permission_changed (priv->permission, NULL, self);
       priv->lock_button = widget;
     }
+  else
+    g_warning ("Your system does not have the cups-pk-helper's policy \
+\"org.opensuse.cupspkhelper.mechanism.all-edit\" installed. \
+Please check your installation");
 
 
   gtk_style_context_get_background_color (gtk_widget_get_style_context (top_widget),
@@ -2403,6 +2592,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
   populate_printers_list (self);
   populate_jobs_list (self);
   populate_allowed_users_list (self);
+  attach_to_cups_notifier (self);
 
   gtk_container_add (GTK_CONTAINER (self), top_widget);
   gtk_widget_show_all (GTK_WIDGET (self));
